// MoveObjects.cpp: implementation of the CMoveObjects class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "frontcad.h"
#include "MoveObjects.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMoveObjects::CMoveObjects()
{
	m_pHead = 0;
	m_pTail = 0;
	m_Ref = CPoint(0, 0);
}

CMoveObjects::~CMoveObjects()
{

}

void CMoveObjects::AddObject(CCadObject *pObj)
{
	if(m_pHead)
	{
		///--------------------------
		///add object to end of list
		///--------------------------
		m_pTail->m_pNext = pObj;
		pObj->m_pPrev = m_pTail;
		m_pTail = pObj;
	}
	else	///first object
	{
		m_pHead = pObj;
		m_pTail = pObj;
	}
}

CCadObject * CMoveObjects::RemoveObject(CCadObject *pLO)
{
	///------------------------------------------
	/// RemoveObject
	///			This function removes an object
	/// from the move buffer.  This is a DELETE
	///
	///	parameters:
	///		pLO.....pointer to object to remove
	///------------------------------------------
	if(pLO == m_pHead)
	{
		m_pHead = (CCadObject *)m_pHead->m_pNext;
		if(m_pHead) m_pHead->m_pPrev = 0;
	}
	else if (pLO == m_pTail)
	{
		m_pTail = (CCadObject*)m_pTail->m_pPrev;
		if(m_pTail)m_pTail->m_pNext = 0;
	}
	else
	{
		pLO->m_pPrev->m_pNext = pLO->m_pNext;
		pLO->m_pNext->m_pPrev = pLO->m_pPrev;
	}
	pLO->m_pPrev = 0;
	pLO->m_pNext = 0;
	return pLO;
}

void CMoveObjects::Draw(CDC *pDC, CPoint Off, CScale Scale)
{
	CCadObject *pCO = m_pHead;
	while(pCO)
	{
		pCO->Draw(pDC,OBJECT_MODE_SELECTED,Off,Scale);
		pCO = pCO->m_pNext;
	}

}

void CMoveObjects::SetRef(CPoint Ref)
{
	CCadObject *pCO = m_pHead;
	while(pCO)
	{
		pCO->AdjustRefernce(Ref);
		pCO = pCO->m_pNext;
	}
	m_Ref = Ref;
}

void CMoveObjects::Clear(int nDeleteFlag)
{
	CCadObject *pCO = m_pHead;
	CCadObject *pNext;
	while(pCO && nDeleteFlag)
	{
		pNext = pCO->m_pNext;
		delete pCO;
		pCO = pNext;
	}
	this->m_pHead = 0;
	this->m_pTail = 0;
}

// Copy a MoveObjects class
void CMoveObjects::Copy(CMoveObjects & Src)
{
	CCadObject *pObj = Src.GetHead();
	while (pObj)
	{
		AddObject(pObj->CopyObject());
		pObj = pObj->m_pNext;
	}
}


// Removes object from front of list
CCadObject * CMoveObjects::RemoveFromHead()
{
	CCadObject *pObj;

	pObj = m_pHead;
	if (pObj)
	{
		m_pHead = pObj->m_pNext;
		if(m_pHead)
			m_pHead->m_pPrev = 0;
		pObj->m_pNext = 0;
	}
	return pObj;
}


CCadObject * CMoveObjects::RemoveFromTail()
{
	CCadObject *pObj;

	pObj = m_pTail;
	if (pObj)
	{
		m_pTail = pObj->m_pPrev;
		if(m_pTail)
			m_pTail->m_pNext = 0;
		pObj->m_pPrev = 0;
	}
	return pObj;
}
